@@ -43,7 +43,6 @@
 #define IN_PROPERTIES( pos ) buffer->in_string[(pos)].properties
 #define IN_LIGID( pos )      buffer->in_string[(pos)].ligID
 
-#define OUT_GLYPH( pos )     buffer->out_string[(pos)].gindex
 
 #define ADD_String( buffer, num_in, num_out, glyph_data, component, ligID )             \
           ( ( error = otl_buffer_add_output_glyphs( (buffer),                           \
@@ -3206,7 +3205,7 @@
 
       /* check whether context is too long; it is a first guess only */
 
-      if ( bgc > buffer->out_pos || buffer->in_pos + igc + lgc > buffer->in_length )
+      if ( bgc > buffer->in_pos || buffer->in_pos + igc + lgc > buffer->in_length )
 	goto next_chainsubrule;
 
       if ( bgc )
@@ -3214,9 +3213,9 @@
         /* since we don't know in advance the number of glyphs to inspect,
            we search backwards for matches in the backtrack glyph array    */
 
-        for ( i = 0, j = buffer->out_pos - 1; i < bgc; i++, j-- )
+        for ( i = 0, j = buffer->in_pos - 1; i < bgc; i++, j-- )
         {
-          while ( CHECK_Property( gdef, OUT_GLYPH( j ), flags, &property ) )
+          while ( CHECK_Property( gdef, IN_GLYPH( j ), flags, &property ) )
           {
             if ( error && error != TTO_Err_Not_Covered )
               return error;
@@ -3236,7 +3235,7 @@
                Backtrack offsets -  3  2  1  0
                Lookahead offsets -                    0  1  2  3           */
 
-          if ( OUT_GLYPH( j ) != curr_csr.Backtrack[i] )
+          if ( IN_GLYPH( j ) != curr_csr.Backtrack[i] )
 	    goto next_chainsubrule;
         }
       }
@@ -3372,7 +3371,7 @@
 
       /* check whether context is too long; it is a first guess only */
 
-      if ( bgc > buffer->out_pos || buffer->in_pos + igc + lgc > buffer->in_length )
+      if ( bgc > buffer->in_pos || buffer->in_pos + igc + lgc > buffer->in_length )
 	goto next_chainsubclassrule;
 
       if ( bgc )
@@ -3383,9 +3382,9 @@
 
         bc       = ccsr.Backtrack;
 
-        for ( i = 0, j = buffer->out_pos - 1; i < bgc; i++, j-- )
+        for ( i = 0, j = buffer->in_pos - 1; i < bgc; i++, j-- )
         {
-          while ( CHECK_Property( gdef, OUT_GLYPH( j ), flags, &property ) )
+          while ( CHECK_Property( gdef, IN_GLYPH( j ), flags, &property ) )
           {
             if ( error && error != TTO_Err_Not_Covered )
               goto End1;
@@ -3399,7 +3398,7 @@
           {
             /* Keeps us from having to do this for each rule */
 
-            error = Get_Class( &ccsf2->BacktrackClassDef, OUT_GLYPH( j ),
+            error = Get_Class( &ccsf2->BacktrackClassDef, IN_GLYPH( j ),
                                &backtrack_classes[i], NULL );
             if ( error && error != TTO_Err_Not_Covered )
               goto End1;
@@ -3527,7 +3526,7 @@
 
     /* check whether context is too long; it is a first guess only */
 
-    if ( bgc > buffer->out_pos || buffer->in_pos + igc + lgc > buffer->in_length )
+    if ( bgc > buffer->in_pos || buffer->in_pos + igc + lgc > buffer->in_length )
       return TTO_Err_Not_Covered;
 
     if ( bgc )
@@ -3537,9 +3536,9 @@
 
       bc       = ccsf3->BacktrackCoverage;
 
-      for ( i = 0, j = buffer->out_pos - 1; i < bgc; i++, j-- )
+      for ( i = 0, j = buffer->in_pos - 1; i < bgc; i++, j-- )
       {
-        while ( CHECK_Property( gdef, OUT_GLYPH( j ), flags, &property ) )
+        while ( CHECK_Property( gdef, IN_GLYPH( j ), flags, &property ) )
         {
           if ( error && error != TTO_Err_Not_Covered )
             return error;
@@ -3549,7 +3548,7 @@
 	  j--;
         }
 
-        error = Coverage_Index( &bc[i], OUT_GLYPH( j ), &index );
+        error = Coverage_Index( &bc[i], IN_GLYPH( j ), &index );
         if ( error )
           return error;
       }
@@ -4060,14 +4059,10 @@
     FT_UShort*   properties;
     FT_UShort*   index;
 
-    /* Each feature can only be added once once */
-    
-    if ( !gsub ||
-         feature_index >= gsub->FeatureList.FeatureCount ||
-	 gsub->FeatureList.ApplyCount == gsub->FeatureList.FeatureCount )
-      return TT_Err_Invalid_Argument;
 
-    gsub->FeatureList.ApplyOrder[gsub->FeatureList.ApplyCount++] = feature_index;
+    if ( !gsub ||
+         feature_index >= gsub->FeatureList.FeatureCount )
+      return TT_Err_Invalid_Argument;
 
     properties = gsub->LookupList.Properties;
 
@@ -4092,8 +4087,6 @@
     if ( !gsub )
       return TT_Err_Invalid_Argument;
 
-    gsub->FeatureList.ApplyCount = 0;
-
     properties = gsub->LookupList.Properties;
 
     for ( i = 0; i < gsub->LookupList.LookupCount; i++ )
@@ -4123,34 +4116,32 @@
 				  OTL_Buffer        buffer )
   {
     FT_Error          error, retError = TTO_Err_Not_Covered;
-    FT_UShort         i, j, feature_index;
-    TTO_Feature       feature;
+    FT_UShort         j;
+
+    FT_UShort*        properties;
 
     if ( !gsub ||
          !buffer || buffer->in_length == 0 || buffer->in_pos >= buffer->in_length )
       return TT_Err_Invalid_Argument;
 
-    for ( i = 0; i < gsub->FeatureList.ApplyCount; i++)
-    {
-      feature_index = gsub->FeatureList.ApplyOrder[i];
-      feature = gsub->FeatureList.FeatureRecord[feature_index].Feature;
-
-      for ( j = 0; j < feature.LookupListCount; j++ )
+    properties = gsub->LookupList.Properties;
+    
+    for ( j = 0; j < gsub->LookupList.LookupCount; j++ )
+      if ( properties[j] )
       {
-        error = Do_String_Lookup( gsub, feature.LookupListIndex[j], buffer );
-	if ( error )
+        error = Do_String_Lookup( gsub, j, buffer );
+        if ( error )
 	{
 	  if ( error != TTO_Err_Not_Covered )
 	    goto End;
 	}
 	else
 	  retError = error;
-        
+	  
 	error = otl_buffer_swap( buffer );
 	if ( error )
 	  goto End;
-      } 
-    }
+      }
     
     error = retError;
 
