@@ -31,16 +31,14 @@
 #include "pangoft2.h"
 #include "pangoft2-private.h"
 #include "pangofc-fontmap.h"
+#include "pangofc-private.h"
 
 /* for compatibility with older freetype versions */
 #ifndef FT_LOAD_TARGET_MONO
 #define FT_LOAD_TARGET_MONO  FT_LOAD_MONOCHROME
 #endif
 
-#define PANGO_TYPE_FT2_FONT              (pango_ft2_font_get_type ())
-#define PANGO_FT2_FONT(object)           (G_TYPE_CHECK_INSTANCE_CAST ((object), PANGO_TYPE_FT2_FONT, PangoFT2Font))
 #define PANGO_FT2_FONT_CLASS(klass)      (G_TYPE_CHECK_CLASS_CAST ((klass), PANGO_TYPE_FT2_FONT, PangoFT2FontClass))
-#define PANGO_FT2_IS_FONT(object)        (G_TYPE_CHECK_INSTANCE_TYPE ((object), PANGO_TYPE_FT2_FONT))
 #define PANGO_FT2_IS_FONT_CLASS(klass)   (G_TYPE_CHECK_CLASS_TYPE ((klass), PANGO_TYPE_FT2_FONT))
 #define PANGO_FT2_FONT_GET_CLASS(obj)    (G_TYPE_INSTANCE_GET_CLASS ((obj), PANGO_TYPE_FT2_FONT, PangoFT2FontClass))
 
@@ -58,10 +56,6 @@ typedef struct
   int bitmap_top;
 } PangoFT2RenderedGlyph;
 
-static PangoFontClass *parent_class;	/* Parent class structure for PangoFT2Font */
-
-static void pango_ft2_font_class_init (PangoFT2FontClass *class);
-static void pango_ft2_font_init       (PangoFT2Font      *ft2font);
 static void pango_ft2_font_finalize   (GObject         *object);
 
 static void                   pango_ft2_font_get_glyph_extents (PangoFont      *font,
@@ -78,8 +72,6 @@ static guint      pango_ft2_font_real_get_glyph         (PangoFcFont      *font,
 static PangoGlyph pango_ft2_font_real_get_unknown_glyph (PangoFcFont      *font,
 							 gunichar          wc);
 
-static GType      pango_ft2_font_get_type               (void);
-
 static void       pango_ft2_get_item_properties         (PangoItem      *item,
 							 PangoUnderline *uline,
 							 gboolean       *strikethrough,
@@ -262,33 +254,7 @@ pango_ft2_font_get_face (PangoFont *font)
   return ft2font->face;
 }
 
-static GType
-pango_ft2_font_get_type (void)
-{
-  static GType object_type = 0;
-
-  if (!object_type)
-    {
-      static const GTypeInfo object_info =
-      {
-        sizeof (PangoFT2FontClass),
-        (GBaseInitFunc) NULL,
-        (GBaseFinalizeFunc) NULL,
-        (GClassInitFunc) pango_ft2_font_class_init,
-        NULL,           /* class_finalize */
-        NULL,           /* class_data */
-        sizeof (PangoFT2Font),
-        0,              /* n_preallocs */
-        (GInstanceInitFunc) pango_ft2_font_init,
-      };
-      
-      object_type = g_type_register_static (PANGO_TYPE_FC_FONT,
-                                            "PangoFT2Font",
-                                            &object_info, 0);
-    }
-  
-  return object_type;
-}
+G_DEFINE_TYPE (PangoFT2Font, pango_ft2_font, PANGO_TYPE_FC_FONT)
 
 static void 
 pango_ft2_font_init (PangoFT2Font *ft2font)
@@ -307,8 +273,6 @@ pango_ft2_font_class_init (PangoFT2FontClass *class)
   PangoFontClass *font_class = PANGO_FONT_CLASS (class);
   PangoFcFontClass *fc_font_class = PANGO_FC_FONT_CLASS (class);
   
-  parent_class = g_type_class_peek_parent (class);
-  
   object_class->finalize = pango_ft2_font_finalize;
   
   font_class->get_glyph_extents = pango_ft2_font_get_glyph_extents;
@@ -361,11 +325,11 @@ pango_ft2_font_render_glyph (PangoFont *font,
 }
 
 static void
-transform_point (PangoMatrix *matrix,
-		 int          x,
-		 int          y,
-		 int         *x_out_pixels,
-		 int         *y_out_pixels)
+transform_point (const PangoMatrix *matrix,
+		 int                x,
+		 int                y,
+		 int               *x_out_pixels,
+		 int               *y_out_pixels)
 {
   double x_out = (matrix->xx * x + matrix->xy * y) / PANGO_SCALE + matrix->x0;
   double y_out = (matrix->yx * x + matrix->yy * y) / PANGO_SCALE + matrix->y0;
@@ -393,12 +357,12 @@ transform_point (PangoMatrix *matrix,
  * transformation matrix to that passed in to this function.
  **/
 void 
-pango_ft2_render_transformed (FT_Bitmap        *bitmap,
-			      PangoMatrix      *matrix,
-			      PangoFont        *font,
-			      PangoGlyphString *glyphs,
-			      int               x, 
-			      int               y)
+pango_ft2_render_transformed (FT_Bitmap         *bitmap,
+			      const PangoMatrix *matrix,
+			      PangoFont         *font,
+			      PangoGlyphString  *glyphs,
+			      int                x, 
+			      int                y)
 {
   FT_UInt glyph_index;
   int i;
@@ -421,8 +385,8 @@ pango_ft2_render_transformed (FT_Bitmap        *bitmap,
 	  PangoFT2RenderedGlyph *rendered_glyph;
 	  glyph_index = gi->glyph;
 
-	  rendered_glyph = pango_ft2_font_get_cache_glyph_data (font,
-								glyph_index);
+	  rendered_glyph = _pango_ft2_font_get_cache_glyph_data (font,
+								 glyph_index);
 	  add_glyph_to_cache = FALSE;
 	  if (rendered_glyph == NULL)
 	    {
@@ -527,10 +491,10 @@ pango_ft2_render_transformed (FT_Bitmap        *bitmap,
 
 	  if (add_glyph_to_cache)
 	    {
-	      pango_ft2_font_set_glyph_cache_destroy (font,
-						      (GDestroyNotify) pango_ft2_free_rendered_glyph);
-	      pango_ft2_font_set_cache_glyph_data (font,
-						   glyph_index, rendered_glyph);
+	      _pango_ft2_font_set_glyph_cache_destroy (font,
+						       (GDestroyNotify) pango_ft2_free_rendered_glyph);
+	      _pango_ft2_font_set_cache_glyph_data (font,
+						    glyph_index, rendered_glyph);
 	    }
 	}
 
@@ -558,72 +522,25 @@ pango_ft2_render (FT_Bitmap        *bitmap,
   pango_ft2_render_transformed (bitmap, NULL, font, glyphs, x * PANGO_SCALE, y * PANGO_SCALE);
 }
 
-static FT_Glyph_Metrics *
-pango_ft2_get_per_char (PangoFont *font,
-			guint32    glyph_index)
-{
-  PangoFT2Font *ft2font = (PangoFT2Font *)font;
-  FT_Face face;
-	
-  face = pango_ft2_font_get_face (font);
-  
-  FT_Load_Glyph (face, glyph_index, ft2font->load_flags);
-  return &face->glyph->metrics;
-}
-
 static PangoFT2GlyphInfo *
 pango_ft2_font_get_glyph_info (PangoFont   *font,
 			       PangoGlyph   glyph,
 			       gboolean     create)
 {
   PangoFT2Font *ft2font = (PangoFT2Font *)font;
+  PangoFcFont *fcfont = (PangoFcFont *)font;
   PangoFT2GlyphInfo *info;
-  FT_Glyph_Metrics *gm;
 
   info = g_hash_table_lookup (ft2font->glyph_info, GUINT_TO_POINTER (glyph));
 
   if ((info == NULL) && create)
     {
-      FT_Face face = pango_ft2_font_get_face (font);
-      info = g_new0 (PangoFT2GlyphInfo, 1);
-      
-      if (glyph && (gm = pango_ft2_get_per_char (font, glyph)))
-	{
-	  info->ink_rect.x = PANGO_UNITS_26_6 (gm->horiBearingX);
-	  info->ink_rect.width = PANGO_UNITS_26_6 (gm->width);
-	  info->ink_rect.y = -PANGO_UNITS_26_6 (gm->horiBearingY);
-	  info->ink_rect.height = PANGO_UNITS_26_6 (gm->height);
-	      
-	  info->logical_rect.x = 0;
-	  info->logical_rect.width = PANGO_UNITS_26_6 (gm->horiAdvance);
-	  if (ft2font->load_flags & FT_LOAD_NO_HINTING)
-	    {
-	      FT_Fixed ascender, descender;
+       info = g_new0 (PangoFT2GlyphInfo, 1);
 
-	      ascender = FT_MulFix (face->ascender, face->size->metrics.y_scale);
-	      descender = FT_MulFix (face->descender, face->size->metrics.y_scale);
-
-	      info->logical_rect.y = - PANGO_UNITS_26_6 (ascender);
-	      info->logical_rect.height = PANGO_UNITS_26_6 (ascender - descender);
-	    }
-	  else
-	    {
-	      info->logical_rect.y = - PANGO_UNITS_26_6 (face->size->metrics.ascender);
-	      info->logical_rect.height = PANGO_UNITS_26_6 (face->size->metrics.ascender - face->size->metrics.descender);
-	    }
-	}
-      else
-	{
-	  info->ink_rect.x = 0;
-	  info->ink_rect.width = 0;
-	  info->ink_rect.y = 0;
-	  info->ink_rect.height = 0;
-
-	  info->logical_rect.x = 0;
-	  info->logical_rect.width = 0;
-	  info->logical_rect.y = 0;
-	  info->logical_rect.height = 0;
-	}
+      pango_fc_font_get_raw_extents (fcfont, ft2font->load_flags,
+				     glyph,
+				     &info->ink_rect,
+				     &info->logical_rect);
 
       g_hash_table_insert (ft2font->glyph_info, GUINT_TO_POINTER(glyph), info);
     }
@@ -766,7 +683,7 @@ pango_ft2_font_finalize (GObject *object)
 			       pango_ft2_free_glyph_info_callback, object);
   g_hash_table_destroy (ft2font->glyph_info);
   
-  G_OBJECT_CLASS (parent_class)->finalize (object);
+  G_OBJECT_CLASS (pango_ft2_font_parent_class)->finalize (object);
 }
 
 /**
@@ -804,7 +721,7 @@ pango_ft2_get_unknown_glyph (PangoFont *font)
 
 static void
 draw_underline (FT_Bitmap         *bitmap,
-		PangoMatrix       *matrix,
+		const PangoMatrix *matrix,
 		PangoFontMetrics  *metrics,
 		PangoUnderline     uline,
 		int                x,
@@ -860,12 +777,12 @@ draw_underline (FT_Bitmap         *bitmap,
 }
 
 static void
-draw_strikethrough (FT_Bitmap        *bitmap,
-		    PangoMatrix      *matrix,
-		    PangoFontMetrics *metrics,
-		    int               x,
-		    int               width,
-		    int               base_y)
+draw_strikethrough (FT_Bitmap         *bitmap,
+		    const PangoMatrix *matrix,
+		    PangoFontMetrics  *metrics,
+		    int                x,
+		    int                width,
+		    int                base_y)
 {
   int strikethrough_thickness = pango_font_metrics_get_strikethrough_thickness (metrics);
   int strikethrough_position = pango_font_metrics_get_strikethrough_position (metrics);
@@ -900,7 +817,7 @@ pango_ft2_render_layout_line_subpixel (FT_Bitmap       *bitmap,
   PangoRectangle logical_rect;
   PangoRectangle ink_rect;
   int x_off = 0;
-  PangoMatrix *matrix;
+  const PangoMatrix *matrix;
 
   matrix = pango_context_get_matrix (pango_layout_get_context (line->layout));
 
@@ -1147,7 +1064,7 @@ _pango_ft2_ft_strerror (FT_Error error)
 
 
 void *
-pango_ft2_font_get_cache_glyph_data (PangoFont *font,
+_pango_ft2_font_get_cache_glyph_data (PangoFont *font,
 				     int        glyph_index)
 {
   PangoFT2GlyphInfo *info;
@@ -1163,7 +1080,7 @@ pango_ft2_font_get_cache_glyph_data (PangoFont *font,
 }
 
 void
-pango_ft2_font_set_cache_glyph_data (PangoFont     *font,
+_pango_ft2_font_set_cache_glyph_data (PangoFont     *font,
 				     int            glyph_index,
 				     void          *cached_glyph)
 {
@@ -1179,8 +1096,8 @@ pango_ft2_font_set_cache_glyph_data (PangoFont     *font,
 }
 
 void
-pango_ft2_font_set_glyph_cache_destroy (PangoFont      *font,
-					GDestroyNotify  destroy_notify)
+_pango_ft2_font_set_glyph_cache_destroy (PangoFont      *font,
+					 GDestroyNotify  destroy_notify)
 {
   g_return_if_fail (PANGO_FT2_IS_FONT (font));
   
