@@ -27,13 +27,14 @@
 #include "pango-context.h"
 #include "pango-utils.h"
 #include "pango-engine.h"
+#include "pango-enum-types.h"
 
 #include <errno.h>
 #include <string.h>
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
-#include <stdio.h>
+#include <glib/gprintf.h>
 
 #if USE_LA_MODULES
 #define SOEXT ".la"
@@ -84,12 +85,45 @@ escape_string (const char *str)
   return g_string_free (result, FALSE);
 }
 
+/* Suppresses strict-aliasing warnings for gcc >= 3.3 */
+#if __GNUC__ > 3 || (__GNUC__ == 3 && __GNUC_MINOR__ >= 3)
+#define GET_SYMBOL(module,name,location) ({              \
+  gboolean result;					 \
+  void *tmp;                                             \
+  result = g_module_symbol (module, name, &tmp);         \
+  location = (typeof(location))tmp;                      \
+  result;                                                \
+})
+#else
+#define GET_SYMBOL(module,name,location)                 \
+  g_module_symbol (module, name, (gpointer *)&location)
+#endif
+
+static const char *
+script_from_string (PangoScript script)
+{
+  static GEnumClass *class = NULL;
+  GEnumValue *value;
+  if (!class)
+    class = g_type_class_ref (PANGO_TYPE_SCRIPT);
+  
+  value = g_enum_get_value (class, script);
+  if (!value)
+    {
+      g_warning ("Engine reported invalid script value %d\n", script);
+      return script_from_string (PANGO_SCRIPT_INVALID_CODE);
+    }
+
+  return value->value_nick;
+}
+
 void 
 query_module (const char *dir, const char *name)
 {
   void (*list) (PangoEngineInfo **engines, gint *n_engines);
-  PangoEngine *(*load) (const gchar *id);
-  void (*unload) (PangoEngine *engine);
+  void (*init) (GTypeModule *module);
+  void (*exit) (void);
+  PangoEngine *(*create) (const gchar *id);
 
   GModule *module;
   gchar *path;
@@ -102,12 +136,13 @@ query_module (const char *dir, const char *name)
   module = g_module_open (path, 0);
 
   if (!module)
-    fprintf(stderr, "Cannot load module %s: %s\n", path, g_module_error());
+    g_printerr ("Cannot load module %s: %s\n", path, g_module_error ());
 	  
   if (module &&
-      g_module_symbol (module, "script_engine_list", (gpointer *) &list) &&
-      g_module_symbol (module, "script_engine_load", (gpointer *) &load) &&
-      g_module_symbol (module, "script_engine_unload", (gpointer *) &unload))
+      GET_SYMBOL (module, "script_engine_list", list) &&
+      GET_SYMBOL (module, "script_engine_init", init) &&
+      GET_SYMBOL (module, "script_engine_exit", exit) &&
+      GET_SYMBOL (module, "script_engine_create", create))
     {
       gint i,j;
       PangoEngineInfo *engines;
@@ -131,25 +166,24 @@ query_module (const char *dir, const char *name)
 	      quoted_path = g_strdup (path);
 	    }
 	  
-	  printf ("%s%s%s %s %s %s ", quote, quoted_path, quote,
-		  engines[i].id, engines[i].engine_type, engines[i].render_type);
+	  g_printf ("%s%s%s %s %s %s ", quote, quoted_path, quote,
+		    engines[i].id, engines[i].engine_type, engines[i].render_type);
 	  g_free (quoted_path);
 
-	  for (j=0; j < engines[i].n_ranges; j++)
+	  for (j=0; j < engines[i].n_scripts; j++)
 	    {
 	      if (j != 0)
-		printf (" ");
-	      printf ("%d-%d:%s",
-		      engines[i].ranges[j].start,
-		      engines[i].ranges[j].end,
-		      engines[i].ranges[j].langs);
+		g_printf (" ");
+	      g_printf ("%s:%s",
+			script_from_string (engines[i].scripts[j].script),
+			engines[i].scripts[j].langs);
 	    }
-	  printf ("\n");
+	  g_printf ("\n");
 	}
     }
   else
     {
-      fprintf (stderr, "%s does not export Pango module API\n", path);
+      g_printerr ("%s does not export Pango module API\n", path);
     }
   
   g_free (path);
@@ -163,9 +197,11 @@ int main (int argc, char **argv)
   int i;
   char *path;
 
-  printf ("# Pango Modules file\n"
-	  "# Automatically generated file, do not edit\n"
-	  "#\n");
+  g_type_init ();
+  
+  g_printf ("# Pango Modules file\n"
+	    "# Automatically generated file, do not edit\n"
+	    "#\n");
 
   if (argc == 1)		/* No arguments given */
     {
@@ -179,7 +215,7 @@ int main (int argc, char **argv)
 				 "modules",
 				 NULL);
 
-      printf ("# ModulesPath = %s\n#\n", path);
+      g_printf ("# ModulesPath = %s\n#\n", path);
 
       dirs = pango_split_file_list (path);
 
